import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:webview_flutter/webview_flutter.dart';

typedef ZaptWebViewCreatedCallback = void Function(ZaptWebViewController controller);

const String viewType = 'zapt_web_view';

class ZaptWebView extends StatelessWidget {
  const ZaptWebView({
    super.key,
    this.initialURL,
    this.activeTTS,
    this.injectedJavaScript,
    this.onZaptWebViewCreated,
    this.javascriptChannels,
  });

  final String? initialURL;
  final String? injectedJavaScript;
  final bool? activeTTS;
  final ZaptWebViewCreatedCallback? onZaptWebViewCreated;
  final Set<JavascriptChannel>? javascriptChannels;

  @override
  Widget build(BuildContext context) {
    Map<String, dynamic> creationParams = <String, dynamic>{
      "initialURL": initialURL ?? "",
      "injectedJavaScript": injectedJavaScript ?? "",
      "activeTTS": activeTTS ?? false,
      "javascriptChannels": getJavascriptChannelsNames()
    };

    return PlatformViewLink(
      viewType: viewType,
      surfaceFactory: (context, controller) {
        return AndroidViewSurface(
          controller: controller as AndroidViewController,
          gestureRecognizers: const <Factory<OneSequenceGestureRecognizer>>{},
          hitTestBehavior: PlatformViewHitTestBehavior.opaque,
        );
      },
      onCreatePlatformView: (params) {
        return PlatformViewsService.initSurfaceAndroidView(
          id: params.id,
          viewType: viewType,
          layoutDirection: TextDirection.ltr,
          creationParams: creationParams,
          creationParamsCodec: const StandardMessageCodec(),
          onFocus: () {
            params.onFocusChanged(true);
          },
        )
          ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
          ..addOnPlatformViewCreatedListener(
              (id) => _onPlatformViewCreated(id))
          ..create();
      },
    );
  }

  void _onPlatformViewCreated(int id) {
    if (onZaptWebViewCreated != null) {
      onZaptWebViewCreated!(ZaptWebViewController._(id));
      initializeJavascriptChannelsListen();
    }
  }

  List<String> getJavascriptChannelsNames() {
    List<String> javascriptChannelsNames = [];
    if(javascriptChannels != null) {
      for(JavascriptChannel javascriptChannel in javascriptChannels!) {
        javascriptChannelsNames.add(javascriptChannel.name);
      }
    }
    return javascriptChannelsNames; 
  }

  void initializeJavascriptChannelsListen() {
    if(javascriptChannels != null) {
      for(JavascriptChannel javascriptChannel in javascriptChannels!) {
        String channelName = javascriptChannel.name;
        BasicMessageChannel channel = BasicMessageChannel(channelName, const StringCodec());
        channel.setMessageHandler((message) async {
          if(message != null) {
            javascriptChannel.onMessageReceived(JavascriptMessage(message));
          }
          return "";
        });
      }
    }
  }
}

class ZaptWebViewController {
  ZaptWebViewController._(int id)
      : _channel = MethodChannel("zapt_web_view_$id");

  final MethodChannel _channel;

  Future<void> loadUrl(String url) {
    return _channel.invokeListMethod('loadURL', url);
  }

  Future<void> runJavascript(String script) {
    return _channel.invokeListMethod("runJavascript", script);
  }
}
